from django.apps import AppConfig


class WagtailContentImportMicrosoftPickerAppConfig(AppConfig):
    name = "wagtail_content_import.pickers.microsoft"
    label = "wagtail_content_import_microsoft_picker"
    verbose_name = "Wagtail Content Import - Microsoft Picker"



from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from .. import Picker
from ...parsers import get_docx_parser


def parse_document(document):
    parser = get_docx_parser()
    return parser(document).parse()


class LocalPicker(Picker):
    name = "local"
    verbose_name = "Local file"
    icon = "icon-folder-open-inverse"

    def get_context(self):
        return {
            "picker": self,
        }

    js_template = "wagtail_content_import/local_picker_js_init.html"

    def render_js_init(self, request):
        return mark_safe(
            render_to_string(self.js_template, self.get_context(), request=request)
        )

    class Media:
        css = {}
        js = []



import json
import os

from django.test import TestCase

from .google import GoogleDocumentParser


class TestGoogleDocumentParser(TestCase):
    def setUp(self):
        with open(
            os.path.join(os.path.dirname(__file__), "test_document.json"), "r"
        ) as test_document_file:
            test_document = json.load(test_document_file)
            self.parser = GoogleDocumentParser(test_document)

    def test_plain_paragraph_to_html(self):
        plain_paragraph = self.parser.document["body"]["content"][5]["paragraph"]
        parsed_paragraph = self.parser.elements_to_html(plain_paragraph, outer_tag="p")
        self.assertEqual(parsed_paragraph["html"], "<p>A plain text paragraph</p>")

    def test_bold_italic_paragraph_to_html(self):
        styled_paragraph = self.parser.document["body"]["content"][7]["paragraph"]
        parsed_paragraph = self.parser.elements_to_html(styled_paragraph, outer_tag="p")
        self.assertEqual(
            parsed_paragraph["html"],
            "<p>A paragraph with <b>bold</b> and <i>italics</i></p>",
        )

    def test_process_table(self):
        table = self.parser.document["body"]["content"][10]["table"]
        processed_table = self.parser.process_table(table)
        text_table = [[cell.get_text() for cell in row] for row in processed_table.rows]
        self.assertEqual(text_table, [["a", "b"]])

    def test_process_list(self):
        list_items = [
            {"html": "A", "level": 0, "list_id": "kix.ekbd3zuridq8"},
            {"html": "B", "level": 0, "list_id": "kix.ekbd3zuridq8"},
        ]
        html = self.parser.process_list(list_items)
        self.assertEqual(html, "<ol><li>A</li><li>B</li></ol>")

    def test_process_image(self):
        embed = self.parser.process_embedded_object("kix.xrypng5vuptx")
        self.assertEqual(
            embed,
            {
                "type": "image",
                "title": "",
                "value": "https://lh5.googleusercontent.com/dmK7P36vTNwm4Q4pdHO5uofPH6hrnAyX5wtMOO-X09VIs9eTefWBWBdvJmUiVfOzc4TsqbDgouCyah-fvzCXbWcm1PvEjRUoxXgdOte1-G23ccLE_-JX0ZU_tbyXqPdQn3Z4uDcJ5PL8ipN9VA",
            },
        )



# Generated by Django 2.1.8 on 2019-10-09 13:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_content_import", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(model_name="oauthcredentials", name="user",),
        migrations.DeleteModel(name="OAuthCredentials",),
    ]



class BaseMapper:
    """
    Base mapper to take a parser output (intermediate_stream) and map it to the desired input format on self.map()
    """


def map(self, intermediate_stream):
    raise NotImplementedError



from .mappers import get_default_mapper
from django.utils.text import slugify


class ContentImportMixin:
    """
    Mixin to allow a Page model to import content (currently from Google)
    """

    can_import = True

    mapper_class = get_default_mapper()

    @classmethod
    def create_from_import(cls, parsed_doc, user):
        """
        Factory method to create the Page and populate it from a parsed document.
        """
        title = parsed_doc["title"]
        mapper_class = cls.mapper_class
        mapper = mapper_class()
        imported_data = mapper.map(parsed_doc["elements"], user=user)
        return cls(title=title, slug=slugify(title), body=imported_data, owner=user,)



from django.apps import AppConfig


class WagtailContentImportLocalPickerAppConfig(AppConfig):
    name = "wagtail_content_import.pickers.local"
    label = "wagtail_content_import_local_picker"
    verbose_name = "Wagtail Content Import - Local File Picker"



from unittest.mock import MagicMock

from django.contrib.auth.models import User
from django.test import TestCase

from wagtail.images import get_image_model
from wagtail.images.tests.utils import get_test_image_file

from .converters import ImageConverter, RichTextConverter, TextConverter, TableConverter
from ..parsers.tables import Table, Cell


class TestConverters(TestCase):
    def test_text_conversion(self):
        text_converter = TextConverter("test_block")
        test_element = {"type": "html", "value": "test_text"}
        converted_element = text_converter(test_element)
        self.assertEqual(
            converted_element,
            ("test_block", "test_text"),
            "Should be: ('test_block', 'test_text'",
        )

    def test_plain_rich_text_conversion(self):
        text_converter = RichTextConverter("test_block")
        test_element = {"type": "html", "value": "test_text"}
        converted_element = text_converter(test_element)
        self.assertEqual(converted_element[0], "test_block", "Should be: 'test_block'")
        self.assertEqual(
            converted_element[1].source, "<p>test_text</p>", "Should be: 'test_text"
        )

    def test_rich_text_script_tags_removed(self):
        text_converter = RichTextConverter("test_block")
        test_element = {"type": "html", "value": "<script>test_text</script>"}
        converted_element = text_converter(test_element)
        self.assertNotIn(
            "script", converted_element[1].source, "Should not contain script tags."
        )

    def test_table_conversion(self):
        table_converter = TableConverter("test_block")
        test_table = Table([[Cell("0"), Cell("1")], [Cell("2"), Cell("3")]])
        test_element = {"type": "table", "value": test_table}
        converted_element = table_converter(test_element)
        self.assertEqual(converted_element[0], "test_block", "Should be 'test_block'")
        self.assertEqual(
            converted_element[1],
            {"data": [["0", "1"], ["2", "3"]]},
            "Should be: {'data': [['0', '1'], ['2', '3']]}",
        )

    def test_image_conversion(self):
        image_converter = ImageConverter("test_block")
        test_image_file = get_test_image_file()
        test_image_file.seek(0)
        image_converter.fetch_image = MagicMock(
            return_value=("content_import_test_image", test_image_file.read())
        )
        test_user = User.objects.create_user(
            username="tester", email="test@test", password="top_secret"
        )
        converted_element = image_converter(
            {"type": "image", "value": "url"}, user=test_user
        )
        Image = get_image_model()
        self.assertEqual(converted_element[0], "test_block", "Should be 'test_block'")
        self.assertIsInstance(converted_element[1], Image)
        self.assertIn(
            converted_element[1],
            Image.objects.filter(title="content_import_test_image"),
        )



from docx import Document

from .base import DocumentParser
from django.utils.html import format_html
from django.utils.safestring import mark_safe


class DocxParser(DocumentParser):
    """Default DocumentParser for docx taking a BytesIO docx file and converting upon parse() into a list of
    {'type': type, 'value': value} intermediate elements"""

    def __init__(self, document):
        self.document = Document(document)

    def generate_simple_tag(self, content, tag):
        return (
            format_html("<{tag}>{content}</{tag}>", tag=tag, content=content)
            if content
            else ""
        )

    def paragraph_to_html(self, paragraph, outer_tag="p"):
        """
        Compile a paragraph into a HTML string, optionally with semantic markup for styles.
        Returns a dictionary of the form:
        {
            'type': 'html',
            'value': html
        }
        """
        text_list = []
        for run in paragraph.runs:
            text = run.text
            if run.bold:
                text = self.generate_simple_tag(text, "b")
            if run.italic:
                text = self.generate_simple_tag(text, "em")
            text_list.append(text)

        content = mark_safe("".join(text_list))

        return {"type": "html", "value": self.generate_simple_tag(content, outer_tag)}

    def parse(self):
        """
        Parse the document and return a set of intermediate {'type': type, 'value': value} blocks that represent it.
        """
        blocks = []

        title = self.document.core_properties.title

        for paragraph in self.document.paragraphs:
            if paragraph.style.name == "Heading 1":
                converted_block = {"type": "heading", "value": paragraph.text}
                if not title:
                    title = paragraph.text
            elif paragraph.style.name[:-1] == "Heading ":
                converted_block = self.paragraph_to_html(
                    paragraph, outer_tag="h" + paragraph.style.name[-1]
                )
            else:
                converted_block = self.paragraph_to_html(paragraph)
            if converted_block["value"]:
                blocks.append(converted_block)

        return {"title": title, "elements": blocks}



from django import template
from django.utils.safestring import mark_safe

from wagtail.core import hooks

register = template.Library()


@register.simple_tag(takes_context=True)
def wagtailcontentimport_pickerjs(context):
    pickers = [fn() for fn in hooks.get_hooks("register_content_import_picker") if fn()]

    js_snippets = []

    for picker in pickers:
        js_snippets.extend(picker.media.render_js())
        js_snippets.append(picker.render_js_init(context["request"]))

    return mark_safe("\n".join(js_snippets))


@register.inclusion_tag(
    "wagtail_content_import/picker_buttons.html", takes_context=True
)
def wagtailcontentimport_picker_buttons(context):
    pickers = [fn() for fn in hooks.get_hooks("register_content_import_picker") if fn()]
    if pickers:
        context["default_picker"] = pickers[0]
        if len(pickers) > 1:
            context["picker_options"] = pickers[1:]
    return context



from wagtail.core import hooks

from .utils import LocalPicker, parse_document
from ...utils import create_page_from_import


@hooks.register("register_content_import_picker")
def register_content_import_picker():
    return LocalPicker()


@hooks.register("before_create_page")
def create_from_local_doc(request, parent_page, page_class):
    if "local-doc" in request.FILES:
        parsed_doc = parse_document(request.FILES["local-doc"].file)
        return create_page_from_import(request, parent_page, page_class, parsed_doc)



#!/usr/bin/env python
import io

from setuptools import setup, find_packages

test_requires = [
    # Required for running the tests
    "mock>=1.0.0",
    # For coverage and PEP8 linting
    "coverage>=3.7.0",
    "flake8>=2.2.0",
    # Required for matrix build on Travis
    "tox==3.9.0",
]

with io.open("README.md", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

setup(
    name="wagtail_content_import",
    version="0.4.2",
    description="A module for Wagtail that provides functionality for importing page content from third-party sources.",
    author="Samir Shah, Jacob Topp-Mugglestone, Karl Hobley, Matthew Westcott",
    author_email="jacobtm@torchbox.com",
    url="https://github.com/torchbox/wagtail-content-import",
    packages=find_packages(),
    include_package_data=True,
    license="BSD",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Wagtail :: 2",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
    install_requires=["wagtail>=2.2", "python-docx>=0.8.10"],
    extras_require={"test": test_requires, },
    zip_safe=False,
)



from .base import DocumentParser
from django.utils.html import escape
from .tables import Table, Cell


class GoogleDocumentParser(DocumentParser):
    """Default DocumentParser for Google Docs, taking a JSON Google Doc and converting upon parse() into a list of
    {'type': type, 'value': value} intermediate elements"""

    def __init__(self, document):
        self.document = document

    def elements_to_html(self, paragraph, outer_tag="p"):
        """
        Compile a paragraph into a HTML string, optionally with semantic markup for styles.
        Returns a dictionary of the form:
        {
            'html': 'HTML content',
            'embeds': '[A list of embedded object IDs that were found in the paragraph]'
        }
        """
        output = []
        embeds = []

        for element in paragraph["elements"]:
            if "inlineObjectElement" in element:
                embeds.append(element["inlineObjectElement"]["inlineObjectId"])
                continue

            text_run = element.get("textRun")
            if not text_run:
                continue

            content = text_run["content"].strip("\n")
            if not content:
                continue

            prefixes = []
            suffixes = []

            style = text_run["textStyle"]
            if style.get("bold"):
                prefixes.append("<b>")
                suffixes.append("</b>")
            if style.get("italic"):
                prefixes.append("<i>")
                suffixes.append("</i>")
            if style.get("underline"):
                prefixes.append("<u>")
                suffixes.append("</u>")
            if style.get("baselineOffset") == "SUPERSCRIPT":
                prefixes.append("<sup>")
                suffixes.append("</sup>")
            elif style.get("baselineOffset") == "SUBSCRIPT":
                prefixes.append("<sub>")
                suffixes.append("</sub>")
            if style.get("strikethrough"):
                prefixes.append("<s>")
                suffixes.append("</s>")
            if style.get("link"):
                url = style["link"].get("url")
                # Links without a 'url' field are local bookmark/heading references;
                # skip these as there's no direct equivalent in Wagtail content
                # https://developers.google.com/docs/api/reference/rest/v1/documents#Link
                if url:
                    prefixes.append('<a href="{}">'.format(escape(url)))
                    suffixes.append("</a>")

            suffixes.reverse()
            html = "".join(prefixes) + escape(content) + "".join(suffixes)
            output.append(html)

        if output:
            inner_html = "".join(output)
            html = (
                f"<{outer_tag}>" + inner_html + f"</{outer_tag}>"
                if outer_tag
                else inner_html
            )
        else:
            html = ""
        return {
            "html": html,
            "embeds": embeds,
        }

    def process_embedded_object(self, embed_id):
        embed = self.document["inlineObjects"][embed_id]
        # Currently we only handle images
        image_props = embed["inlineObjectProperties"]["embeddedObject"].get(
            "imageProperties"
        )
        title = embed["inlineObjectProperties"]["embeddedObject"].get("title", "")
        if image_props:
            return {"type": "image", "value": image_props["contentUri"], "title": title}

    def process_list(self, list_items):
        """
        Return HTML for a series of list items.
        Ordered/unordered lists are only respected at the top level.
        """
        list_id = list_items[0]["list_id"]
        list_definition = self.document["lists"][list_id]
        # ordered lists define a glyphType (DECIMAL, ALPHA etc) whereas unordered lists define
        # a constant glyphSymbol
        is_ordered = (
            "glyphType" in list_definition["listProperties"]["nestingLevels"][0]
        )
        list_tag = "ol" if is_ordered else "ul"

        # Construct an annotated list that can be efficiently converted into a nested tree
        parts = []
        previous_level = None
        info = {}
        for item in list_items:
            # Update previous info item
            if previous_level is None or previous_level < item["level"]:
                info["has_children"] = True

            if previous_level is not None and previous_level > item["level"]:
                info["num_to_close"] = previous_level - item["level"]

            info = {"html": item["html"]}
            previous_level = item["level"]
            parts.append(info)

        if previous_level is not None:
            # close last leaf
            info["num_to_close"] = previous_level

        # Generated nested tree
        html = f"<{list_tag}>"
        for part in parts:
            html += "<li>" + part["html"]
            html += f"<{list_tag}>" if part.get("has_children") else "</li>"
            for i in range(0, part.get("num_to_close", 0)):
                html += f"</{list_tag}></li>"
        html += f"</{list_tag}>"
        return html

    def process_table(self, table):
        imported_rows = [
            [Cell(self.get_cell_text(cell)) for cell in row["tableCells"]]
            for row in table["tableRows"]
        ]
        return Table(imported_rows)

    @staticmethod
    def get_cell_text(cell):
        """
        Get table cell text
        """
        text = ""
        for content_element in cell["content"]:
            if "paragraph" not in content_element:
                continue
            for paragraph_element in content_element["paragraph"]["elements"]:
                if "textRun" not in paragraph_element:
                    continue
                text += paragraph_element["textRun"]["content"]
        return text.strip()

    def parse(self):
        """
        Parse the document and return a set of intermediate {'type': type, 'value': value} blocks that represent it.
        """
        current_block = []
        unprocessed_embeds = []
        blocks = []

        def close_current_block():
            if current_block:
                blocks.append({"type": "html", "value": "".join(current_block)})
                current_block.clear()
            if unprocessed_embeds:
                for embed_id in unprocessed_embeds:
                    embed = self.process_embedded_object(embed_id)
                    if embed:
                        blocks.append(embed)
                unprocessed_embeds.clear()

        body = self.document["body"]

        # List parsing is rather messy because list items are returned as a
        # series of paragaph objects rather than a nested structure.
        current_list = []

        def close_current_list():
            if current_list:
                html = self.process_list(current_list)
                current_block.append(html)
                current_list.clear()

        for part in body["content"]:
            # Part can contain one of sectionBreak, table, tableOfContents, paragraph
            # Currently we only process paragraph and table
            if "table" in part:
                close_current_block()
                close_current_list()
                table = self.process_table(part["table"])
                blocks.append({"type": "table", "value": table})

            if "paragraph" not in part:
                continue

            paragraph = part["paragraph"]
            style = paragraph["paragraphStyle"].get("namedStyleType")
            if style == "HEADING_1":
                close_current_list()
                close_current_block()
                try:
                    blocks.append(
                        {
                            "type": "heading",
                            "value": paragraph["elements"][0]["textRun"][
                                "content"
                            ].strip(),
                        }
                    )
                except KeyError:
                    pass
            elif "bullet" in paragraph:  # We're in a list
                content = self.elements_to_html(paragraph, outer_tag=None)
                current_list.append(
                    {
                        "html": content["html"],
                        "level": paragraph["bullet"].get("nestingLevel", 0),
                        "list_id": paragraph["bullet"]["listId"],
                    }
                )
                unprocessed_embeds += content["embeds"]
            else:
                close_current_list()

                if paragraph["paragraphStyle"]["namedStyleType"] == "HEADING_2":
                    outer_tag = "h2"
                elif paragraph["paragraphStyle"]["namedStyleType"] == "HEADING_3":
                    outer_tag = "h3"
                elif paragraph["paragraphStyle"]["namedStyleType"] == "HEADING_4":
                    outer_tag = "h4"
                elif paragraph["paragraphStyle"]["namedStyleType"] == "HEADING_5":
                    outer_tag = "h5"
                elif paragraph["paragraphStyle"]["namedStyleType"] == "HEADING_6":
                    outer_tag = "h6"
                else:
                    outer_tag = "p"

                content = self.elements_to_html(paragraph, outer_tag=outer_tag)
                unprocessed_embeds += content["embeds"]
                if content["html"]:
                    current_block.append(content["html"])

                # If any embeds were encountered since the last paragraph processed
                # (including within headings / list items that aren't processed as
                # standard paragraphs), close this block and start a new one so that
                # the embed block is in the correct place in the text
                if unprocessed_embeds:
                    close_current_block()

        close_current_list()
        close_current_block()
        return {"title": self.document["title"], "elements": blocks}



from django.apps import AppConfig


class WagtailContentImportAppConfig(AppConfig):
    name = "wagtail_content_import"
    label = "wagtail_content_import"
    verbose_name = "Wagtail Content Import"



from django.apps import AppConfig


class WagtailContentImportGooglePickerAppConfig(AppConfig):
    name = "wagtail_content_import.pickers.google"
    label = "wagtail_content_import_google_picker"
    verbose_name = "Wagtail Content Import - Google Picker"



import json

from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


from .. import Picker
from ...parsers import get_google_parser


def parse_document(document):
    parser = get_google_parser()
    return parser(document).parse()


class GooglePicker(Picker):
    name = "google"
    verbose_name = "Google Docs"
    icon = "icon-doc-full-inverse"

    def __init__(self, oauth_client_config, picker_api_key):
        self.oauth_client_config = json.loads(oauth_client_config)
        self.picker_api_key = picker_api_key

    @property
    def app_id(self):
        return self.oauth_client_config["web"]["project_id"]

    @property
    def client_id(self):
        return self.oauth_client_config["web"]["client_id"]

    def get_context(self):
        return {
            "picker": self,
            "app_id": self.app_id,
            "client_id": self.client_id,
            "picker_api_key": self.picker_api_key,
        }

    js_template = "wagtail_content_import/google_picker_js_init.html"

    def render_js_init(self, request):
        return mark_safe(
            render_to_string(self.js_template, self.get_context(), request=request)
        )

    class Media:
        css = {}
        js = [
            "https://apis.google.com/js/api.js",
            "wagtail_content_import/google_picker.js",
        ]
